@@ -272,7 +272,6 @@ go_load_pango_attributes_into_buffer_filter (PangoAttribute *attribute,
 					  G_GNUC_UNUSED gpointer data)
 {
 	return ((PANGO_ATTR_FOREGROUND == attribute->klass->type) ||
-		(PANGO_ATTR_UNDERLINE == attribute->klass->type) ||
 		(PANGO_ATTR_RISE == attribute->klass->type));
 }
 
@@ -282,6 +281,7 @@ go_load_pango_attributes_into_buffer_named_filter (PangoAttribute *attribute,
 {
 	return ((PANGO_ATTR_STYLE == attribute->klass->type) ||
 		(PANGO_ATTR_WEIGHT == attribute->klass->type) ||
+		(PANGO_ATTR_UNDERLINE == attribute->klass->type) ||
 		(PANGO_ATTR_STRIKETHROUGH == attribute->klass->type));
 }
 
@@ -318,6 +318,16 @@ go_create_std_tags_for_buffer (GtkTextBuffer *buffer)
 				    "weight-set", TRUE, NULL);
 	gtk_text_buffer_create_tag (buffer, "PANGO_WEIGHT_ULTRAHEAVY", "weight", 1000,
 				    "weight-set", TRUE, NULL);
+	gtk_text_buffer_create_tag (buffer, "PANGO_UNDERLINE_NONE", "underline", PANGO_UNDERLINE_NONE,
+				    "underline-set", TRUE, NULL);
+	gtk_text_buffer_create_tag (buffer, "PANGO_UNDERLINE_SINGLE", "underline", PANGO_UNDERLINE_SINGLE,
+				    "underline-set", TRUE, NULL);
+	gtk_text_buffer_create_tag (buffer, "PANGO_UNDERLINE_DOUBLE", "underline", PANGO_UNDERLINE_DOUBLE,
+				    "underline-set", TRUE, NULL);
+	gtk_text_buffer_create_tag (buffer, "PANGO_UNDERLINE_LOW", "underline", PANGO_UNDERLINE_LOW,
+				    "underline-set", TRUE, NULL);
+	gtk_text_buffer_create_tag (buffer, "PANGO_UNDERLINE_ERROR", "underline", PANGO_UNDERLINE_ERROR,
+				    "underline-set", TRUE, NULL);
 }
 
 static gint
@@ -389,6 +399,24 @@ go_load_pango_attributes_into_buffer (PangoAttrList  *markup, GtkTextBuffer *buf
 						gtk_text_buffer_apply_tag (buffer, tag,
 									   &start_iter, &end_iter);
 						break;
+					case PANGO_ATTR_UNDERLINE:
+						val = ((PangoAttrInt *)attribute)->value;
+						if (val == PANGO_UNDERLINE_NONE)
+							gtk_text_buffer_apply_tag_by_name (buffer,"PANGO_UNDERLINE_NONE",
+											   &start_iter, &end_iter);
+						else if (val == PANGO_UNDERLINE_SINGLE)
+							gtk_text_buffer_apply_tag_by_name (buffer,"PANGO_UNDERLINE_SINGLE",
+											   &start_iter, &end_iter);
+						else if (val == PANGO_UNDERLINE_DOUBLE)
+							gtk_text_buffer_apply_tag_by_name (buffer,"PANGO_UNDERLINE_DOUBLE",
+											   &start_iter, &end_iter);
+						else if (val == PANGO_UNDERLINE_LOW)
+							gtk_text_buffer_apply_tag_by_name (buffer,"PANGO_UNDERLINE_LOW",
+											   &start_iter, &end_iter);
+						else if (val == PANGO_UNDERLINE_ERROR)
+							gtk_text_buffer_apply_tag_by_name (buffer,"PANGO_UNDERLINE_ERROR",
+											   &start_iter, &end_iter);
+						break;
 					case PANGO_ATTR_WEIGHT:
 						val = ((PangoAttrInt *)attribute)->value;
 						if (val < (PANGO_WEIGHT_THIN + PANGO_WEIGHT_ULTRALIGHT)/2)
@@ -465,13 +493,6 @@ go_load_pango_attributes_into_buffer (PangoAttrList  *markup, GtkTextBuffer *buf
 							      NULL);
 						g_free (string);
 						break;
-					case PANGO_ATTR_UNDERLINE:
-						g_object_set (G_OBJECT (tag),
-							      "underline",
-							      ((PangoAttrInt *)attribute)->value,
-							      "underline-set", TRUE,
-							      NULL);
-						break;
 					case PANGO_ATTR_RISE:
 						g_object_set (G_OBJECT (tag),
 							      "rise",
@@ -742,3 +763,263 @@ go_pango_attr_superscript_new (gboolean val)
 
 	return (PangoAttribute *) result;
 }
+
+static int
+go_pango_attr_as_markup_string (PangoAttribute *a, GString *gstr)
+{
+	int spans = 0;
+
+	switch (a->klass->type) {
+	case PANGO_ATTR_FONT_DESC :
+		{
+			char *str = pango_font_description_to_string
+				(((PangoAttrFontDesc *)a)->desc);
+			spans += 1;
+			g_string_append_printf (gstr, "<span font_desc=\"%s\">", str);
+			g_free (str);
+		}
+		break;
+	case PANGO_ATTR_FAMILY :
+		spans += 1;
+		g_string_append_printf (gstr, "<span font_family=\"%s\">",
+					((PangoAttrString *)a)->value);
+		break;
+	case PANGO_ATTR_ABSOLUTE_SIZE :
+	case PANGO_ATTR_SIZE :
+		spans += 1;
+		g_string_append_printf (gstr, "<span font_size=\"%i\">",
+					((PangoAttrSize *)a)->size);
+		break;
+	case PANGO_ATTR_RISE:
+		spans += 1;
+		g_string_append_printf (gstr, "<span rise=\"%i\">",
+					((PangoAttrInt *)a)->value);
+		break;
+	case PANGO_ATTR_STYLE :
+		spans += 1;
+		switch (((PangoAttrInt *)a)->value) {
+		case PANGO_STYLE_ITALIC:
+			g_string_append (gstr, "<span font_style=\"italic\">");
+			break;
+		case PANGO_STYLE_OBLIQUE:
+			g_string_append (gstr, "<span font_style=\"oblique\">");
+			break;
+		case PANGO_STYLE_NORMAL:
+		default:
+			g_string_append (gstr, "<span font_style=\"normal\">");
+			break;
+		}
+		break;
+	case PANGO_ATTR_WEIGHT :
+		spans += 1;
+		g_string_append_printf (gstr, "<span font_weight=\"%i\">",
+					((PangoAttrInt *)a)->value);
+	break;
+	case PANGO_ATTR_STRIKETHROUGH :
+		spans += 1;
+		if (((PangoAttrInt *)a)->value)
+			g_string_append (gstr, "<span strikethrough=\"true\">");
+		else
+			g_string_append (gstr, "<span strikethrough=\"false\">");
+		break;
+	case PANGO_ATTR_UNDERLINE :
+		spans += 1;
+		switch (((PangoAttrInt *)a)->value) {
+		case PANGO_UNDERLINE_SINGLE:
+			g_string_append (gstr, "<span underline=\"single\">");
+			break;
+		case PANGO_UNDERLINE_DOUBLE:
+			g_string_append (gstr, "<span underline=\"double\">");
+			break;
+		case PANGO_UNDERLINE_LOW:
+			g_string_append (gstr, "<span underline=\"low\">");
+			break;
+		case PANGO_UNDERLINE_ERROR:
+			g_string_append (gstr, "<span underline=\"error\">");
+			break;
+		case PANGO_UNDERLINE_NONE:
+		default:
+			g_string_append (gstr, "<span underline=\"none\">");
+			break;
+		}
+		break;
+	case PANGO_ATTR_LANGUAGE :
+		spans += 1;
+		g_string_append_printf (gstr, "<span lang=\"%s\">",
+					pango_language_to_string (((PangoAttrLanguage *)a)->value));
+		break;
+	case PANGO_ATTR_VARIANT :
+		spans += 1;
+		if (((PangoAttrInt *)a)->value == PANGO_VARIANT_NORMAL)
+			g_string_append (gstr, "<span font_variant=\"normal\">");
+		else
+			g_string_append (gstr, "<span font_variant=\"smallcaps\">");
+		break;
+	case PANGO_ATTR_LETTER_SPACING :
+		spans += 1;
+		g_string_append_printf (gstr, "<span letter_spacing=\"%i\">",
+					((PangoAttrInt *)a)->value);
+		break;
+	case PANGO_ATTR_FALLBACK :
+		spans += 1;
+		if (((PangoAttrInt *)a)->value)
+			g_string_append (gstr, "<span fallback=\"true\">");
+		else
+			g_string_append (gstr, "<span fallback=\"false\">");
+		break;
+	case PANGO_ATTR_STRETCH :
+		spans += 1;
+		switch (((PangoAttrInt *)a)->value) {
+		case PANGO_STRETCH_ULTRA_CONDENSED:
+			g_string_append (gstr, "<span font_stretch=\"ultracondensed\">");
+			break;
+		case PANGO_STRETCH_EXTRA_CONDENSED:
+			g_string_append (gstr, "<span font_stretch=\"extracondensed\">");
+			break;
+		case PANGO_STRETCH_CONDENSED:
+			g_string_append (gstr, "<span font_stretch=\"condensed\">");
+			break;
+		case PANGO_STRETCH_SEMI_CONDENSED:
+			g_string_append (gstr, "<span font_stretch=\"semicondensed\">");
+			break;
+		case PANGO_STRETCH_SEMI_EXPANDED:
+			g_string_append (gstr, "<span font_stretch=\"semiexpanded\">");
+			break;
+		case PANGO_STRETCH_EXPANDED:
+			g_string_append (gstr, "<span font_stretch=\"expanded\">");
+			break;
+		case PANGO_STRETCH_EXTRA_EXPANDED:
+			g_string_append (gstr, "<span font_stretch=\"extraexpanded\">");
+			break;
+		case PANGO_STRETCH_ULTRA_EXPANDED:
+			g_string_append (gstr, "<span font_stretch=\"ultraexpanded\">");
+			break;
+		case PANGO_STRETCH_NORMAL:
+		default:
+			g_string_append (gstr, "<span font_stretch=\"normal\">");
+			break;
+		}
+		break;
+	case PANGO_ATTR_GRAVITY :
+		spans += 1;
+		switch (((PangoAttrInt *)a)->value) {
+		case PANGO_GRAVITY_SOUTH:
+			g_string_append (gstr, "<span gravity=\"south\">");
+			break;
+		case PANGO_GRAVITY_EAST:
+			g_string_append (gstr, "<span gravity=\"east\">");
+			break;
+		case PANGO_GRAVITY_NORTH:
+			g_string_append (gstr, "<span gravity=\"north\">");
+			break;
+		case PANGO_GRAVITY_WEST:
+			g_string_append (gstr, "<span gravity=\"west\">");
+			break;
+		case PANGO_GRAVITY_AUTO:
+		default:
+			g_string_append (gstr, "<span gravity=\"auto\">");
+			break;
+		}
+		break;
+	case PANGO_ATTR_GRAVITY_HINT :
+		spans += 1;
+		switch (((PangoAttrInt *)a)->value) {
+		case PANGO_GRAVITY_HINT_LINE:
+			g_string_append (gstr, "<span gravity_hint=\"line\">");
+			break;
+		case PANGO_GRAVITY_HINT_STRONG:
+			g_string_append (gstr, "<span gravity_hint=\"strong\">");
+			break;
+		case PANGO_GRAVITY_HINT_NATURAL:
+		default:
+			g_string_append (gstr, "<span gravity_hint=\"natural\">");
+			break;
+		}
+		break;
+
+	case PANGO_ATTR_FOREGROUND :
+		{
+			PangoColor *color = &((PangoAttrColor *)a)->color;
+			spans += 1;
+			g_string_append_printf (gstr, "<span foreground=\"#%02X%02X%02X\">",
+						color->red, color->green, color->blue);
+		}
+		break;
+	case PANGO_ATTR_BACKGROUND :
+		{
+			PangoColor *color = &((PangoAttrColor *)a)->color;
+			spans += 1;
+			g_string_append_printf (gstr, "<span background=\"#%02X%02X%02X\">",
+						color->red, color->green, color->blue);
+		}
+		break;
+	case PANGO_ATTR_UNDERLINE_COLOR :
+		{
+			PangoColor *color = &((PangoAttrColor *)a)->color;
+			spans += 1;
+			g_string_append_printf (gstr, "<span underline_color=\"#%02X%02X%02X\">",
+						color->red, color->green, color->blue);
+		}
+		break;
+	case PANGO_ATTR_STRIKETHROUGH_COLOR :
+		{
+			PangoColor *color = &((PangoAttrColor *)a)->color;
+			spans += 1;
+			g_string_append_printf (gstr, "<span strikethrough_color=\"#%02X%02X%02X\">",
+						color->red, color->green, color->blue);
+		}
+		break;
+
+	case PANGO_ATTR_SCALE :
+	case PANGO_ATTR_SHAPE :
+	default :
+		break; /* ignored */
+	}
+
+	return spans;
+}
+
+char *
+go_pango_attrs_to_markup (PangoAttrList *attrs, char const *text)
+{
+	PangoAttrIterator * iter;
+	int handled = 0;
+	int from, to;
+	int len;
+	GString *gstr;
+
+	if (text == NULL)
+		return NULL;
+	if (attrs == NULL || go_pango_attr_list_is_empty (attrs))
+		return g_strdup (text);
+
+	len = strlen (text);
+	gstr = g_string_sized_new (len + 1);
+
+	iter = pango_attr_list_get_iterator (attrs);
+	do {
+		GSList *list, *l;
+		int spans = 0;
+
+		pango_attr_iterator_range (iter, &from, &to);
+		to = (to > len) ? len : to;       /* Since "to" can be really big! */
+		from = (from > len) ? len : from; /* Since "from" can also be really big! */
+		if (from > handled)
+			g_string_append_len (gstr, text + handled, from - handled);
+		list = pango_attr_iterator_get_attrs (iter);
+		for (l = list; l != NULL; l = l->next)
+			spans += go_pango_attr_as_markup_string (l->data, gstr);
+		g_slist_free (list);
+		if (to > from)
+			g_string_append_len (gstr, text + from, to - from);
+		while (spans-- > 0)
+			g_string_append (gstr, "</span>");
+		handled = to;
+	} while (pango_attr_iterator_next (iter));
+
+	pango_attr_iterator_destroy (iter);
+
+	return g_string_free (gstr, FALSE);
+}
+
+
